import { world, StructureSaveMode, ItemLockMode } from '@minecraft/server';
import { Vector } from './utils';
export class Backpack {
    id;
    slot;
    constructor(data) {
        let ID = typeof data === 'number' ? data : data.getDynamicProperty('ID');
        if (typeof data !== 'number') {
            this.slot = data;
            if (!ID)
                ID = Date.now();
            data.setDynamicProperty('ID', ID);
        }
        this.id = ID;
    }
    loadInventory(entity) {
        const { dimension, location } = entity;
        const { structureManager } = world;
        location.y = dimension.heightRange.min + 1;
        entity.setDynamicProperty('ID', this.id);
        const structure = structureManager.get(`t_mbp:${this.id}`);
        if (!structure)
            return;
        const blocks = new Map();
        for (let i = 0; i < structure.size.y; i++) {
            const block = dimension.getBlock(Vector.add(location, { y: i }));
            blocks.set(block, world.structureManager.createFromWorld(`t_mbp:tmp_${JSON.stringify(Object.values(block.location))}`, dimension, block.location, block.location, {
                saveMode: StructureSaveMode.Memory,
                includeEntities: false,
            }));
            block.inventory?.container?.clearAll();
        }
        structureManager.place(structure, dimension, location);
        structureManager.delete(structure);
        const inv = entity.inventory?.container;
        const entries = [...blocks.entries()];
        for (let i = 0; i < blocks.size; i++) {
            const [block, structure_] = entries[i];
            const container = block.inventory?.container;
            for (let j = 0; j < 27; j++) {
                const item = container.getItem(j);
                if (!item)
                    continue;
                inv.setItem(i * 27 + j, item);
            }
            container.clearAll();
            structureManager.place(structure_, dimension, block.location);
            structureManager.delete(structure_);
        }
    }
    saveInventory(entity, slot) {
        const items = new Map();
        const Binv = entity.inventory?.container;
        for (let i = 0; i < Binv.size; i++) {
            const item = Binv.getItem(i);
            items.set(i, item);
        }
        if (!items.size)
            return;
        const { dimension, location } = entity;
        const { structureManager } = world;
        location.y = dimension.heightRange.min + 1;
        const blocks = new Map();
        let barrel = dimension.getBlock(location);
        blocks.set(barrel, structureManager.createFromWorld(`t_mbp:tmp_${JSON.stringify(Object.values(barrel.location))}`, dimension, barrel.location, Vector.add(barrel.location, { y: 1 }), { includeEntities: false, saveMode: StructureSaveMode.Memory }));
        barrel?.setType('barrel');
        let container = barrel?.inventory?.container;
        let size = 0;
        for (let i = 0; i < items.size; i++) {
            const item = items.get(i);
            const slot = i % 27;
            if (i && !slot) {
                barrel = barrel.above();
                blocks.set(barrel, structureManager.createFromWorld(`t_mbp:tmp_${JSON.stringify(Object.values(barrel.location))}`, dimension, barrel.location, Vector.add(barrel.location, { y: 1 }), { includeEntities: false, saveMode: StructureSaveMode.Memory }));
                barrel.setType('barrel');
                container = barrel.inventory?.container;
                size++;
            }
            container.setItem(slot, item);
        }
        const ID = slot?.getDynamicProperty('ID') ?? this.id;
        structureManager.delete(`t_mbp:${ID}`);
        structureManager.createFromWorld(`t_mbp:${ID}`, dimension, location, Vector.add(location, { y: size }), { includeEntities: false, saveMode: StructureSaveMode.World });
        for (const [block, structure] of blocks.entries()) {
            block.inventory?.container?.clearAll();
            world.structureManager.place(structure, dimension, block.location);
            world.structureManager.delete(structure);
        }
    }
}
export function lockBackpacks(player) {
    const inv = player.inventory.container;
    for (let i = 0; i < inv.size; i++) {
        const slot = inv.getSlot(i);
        if (!slot.hasItem() || !slot.typeId.startsWith('t_mbp:multi_backpack'))
            continue;
        slot.lockMode = i === player.selectedSlotIndex ? ItemLockMode.slot : ItemLockMode.inventory;
    }
}
export function unlockBackpacks(player) {
    const inv = player.inventory.container;
    for (let i = 0; i < inv.size; i++) {
        const slot = inv.getSlot(i);
        if (!slot.hasItem() || !slot.typeId.startsWith('t_mbp:multi_backpack'))
            continue;
        slot.lockMode = ItemLockMode.none;
    }
}
