import { system, world } from '@minecraft/server';
import { Backpack, lockBackpacks, unlockBackpacks } from './libraries/Backpack';
import "./libraries/utils";
import { Vector } from './libraries/utils';
const Backpacks = new Map();
world.gameRules.showTags = false;
system.runInterval(() => {
    for (const player of world.getPlayers()) {
        if (!player.isValid())
            return;
        const { dimension, id: playerID } = player;
        const slot = player.inventory.container.getSlot(player.selectedSlotIndex);
        const isBP = slot.hasItem() && slot.typeId.startsWith('t_mbp:multi_backpack');
        if (!isBP && !Backpacks.has(playerID))
            return;
        const old = Backpacks.get(playerID);
        const backpack = old?.backpack ?? new Backpack(slot);
        const entity = dimension.getEntities({ type: 't_mbp:backpack_container', tags: [`t_mbp:${playerID}`] })[0];
        if (!isBP || (entity && entity.getDynamicProperty('ID') !== slot.getDynamicProperty('ID'))) {
            if (!entity?.isValid())
                return;
            backpack.saveInventory(entity, old?.slot);
            Backpacks.delete(playerID);
            unlockBackpacks(player);
            return entity.remove();
        }
        lockBackpacks(player);
        const container = entity ?? setupContainer(player, backpack, slot.nameTag);
        container.teleport(player.getHeadLocation(), { dimension });
        Backpacks.set(playerID, { backpack, slot });
    }
});
export function setupContainer(player, backpack, name) {
    const entity = player.dimension.spawnEntity('t_mbp:backpack_container', Vector.add(player.getHeadLocation(), Vector.multiply(player.getViewDirection(), 0.5)));
    entity.nameTag = name ?? '§r§f§r§f§rBackpack';
    entity.setDynamicProperty('ID', backpack.id);
    entity.addTag(`t_mbp:${player.id}`);
    const tameable = entity.getComponent('tameable');
    tameable.tame(player);
    backpack.loadInventory(entity);
    return entity;
}
